###
##  Functions for producing elements to .html files. 
##
##  Plan: functions return strings as components to a html file which
##  the caller of these functions compile. Could be a table or something.
##  
##  Version: 0.5
##  Author: Stefan Enroth, LCB.
##



###
## History:
## 2004-11-07 Created.
## 2004-11-09 Fixed some bugs, not needed to supply 'data' anymore, possible to specify size of the images.
##            Also the function for generating a 'blank.png' to be used as starter (with text)
## 2004-11-10 Added make.pic.link.
## 2004-11-13 Wrote the GO-image map things. Tested, running.
## 2004-11-15 Included an imagemap for heatmaps. 
## 2004-12-17 Added the 'styleStr' - base style sheet ripped from dwh. just paste in the beginning of your file.
## 2005-01-04 Fixed the makeGOpage to list individual reporters (sorted) and thier occurancies instead of repreats. 
## 2005-01-14    Minor change of where the blanks are generated from pic.table and some extra control arguments to the GO-functions. All optional (backwards compatible)
## 2005-04-05 Added "_V2" versions where needed
## 2005-05-30 (Robin Andersson) Added functions select.form, contents.div, get.style (and helpers)

####
##
## Todo:
## i) rewrite _V2 functions from scratch. ther're as messy as their _V1 relatives.

####
## Globals
###

styleStr <- "<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\"><style type='text/css'> .uname { color: #005040 } .iname { color: #500050 } input.tiny { font-size: 8pt } .version { font-size: 8pt; font-style: italic; color: gray; text-align: center; font-family: Helvetica; } </style> <style type=\"text/css\"> a:active { text-decoration: none; color: #ff5050; } a:link { text-decoration: none; color: #1000d0; } a:visited { text-decoration: none; color: #1000d0; } a.ext:active { text-decoration: none; color: #ff8050; } a.ext:link { text-decoration: none; color: #f05000; } a.ext:visited { text-decoration: none; color: #800000; } body { font-family: Helvetica, sans-serif; background-color: #F8F8E0;font-size: 10pt;font-weight: normal } div { font-family: Helvetica, sans-serif; background-color: #F8F8E0;font-size: 10pt;font-weight: normal }td { font-family: Helvetica, sans-serif; font-size: 10pt }th { font-family: Helvetica, sans-serif;  text-align: left;background-color: #D3D3BC; font-size: 10pt; }input { font-family: Helvetica, sans-serif; font-size: 10pt }select { font-family: Helvetica; font-size: 10pt }select.mono { font-family: monospace; font-size: 10pt }.normal { font-size: 10pt;font-weight: normal } .large { font-size: 11pt } .help { font-style: italic } .special { font-weight: bold } .pagehead { text-align: left; background-color: #F8F8E0; font-size: 13pt } .subhead { text-align: left; font-size: 11pt; font-weight: bold } .subheadact { text-align: left; font-size: 11pt;font-weight: bold; background-color: #ffe0c0; } .red { color: Red; font-size: 11pt }.del { color: #e00050; font-weight: bold } a.popup { color: #007010 } .tiny { font-size: 8pt } .pink { background-color: #ffe0c0 } </style> </head> <body bgcolor=\"#F8F8E0\" text=black>"


####
## Functions
###


## get.blank, generates a empty image with only the given text displayed.
##
## Notes:
##
## Parameters:
## fname:    (string) Name for the image.
## txt:      (string, formatted) Text to printed inside the image.
##
get.blank <- function(fname = "blank.png", height = 7, width = 7, txt = "Move mouse over links to change image.")
{
  bitmap(file = fname,height = height, width = width,type = "png16m")
  plot.new()
  ## this plot defaults to a coordinate system (0,1)x(0,1), place the text in the middle
  text(0.5,0.5,txt)  
  dev.off()
}


## pic.table, produced a table where the images given can be changed. (hrmpff)
##
## Notes: NO error control in length of input vectors. up to caller to check.
##        Side-effect; creates a (blank) image called 'blank.png'(default, can be specified)
##
## Parameters:
## header:    (strings, dataframe) Names for the head of the table. same dimension as the table x rows.
## pics:      (strings, array) Names for the pictures.
## data:      (dataframe,optional) what should be presented in the table apart from the images.
## use.names: (boolean) if TRUE, the names(links) for the images are in the first colum of 'data'
##            othervise the 'suffix' from the 'pics' are stripped and the rest used as names.
## suffix:    (string,optional) suffix of the pictures, only used if 'use.names = FALSE'
## height:    (numeric) Height in pixels of the images displayed
## width:     (numeric) Width in pixels of the images displayed
## blankname: (string,optional) The default (empty) image displayed's name (defaults to 'blank.png').
## mapnames:  (string,array) If the images displayed have imagemaps, give the names of the maps in the order of the pics here.
## maps:      (string,array) The maps as created by make.-.map below.
## links:     (string,array) If not NULL, the links of the mouseOver will open these instead of the default, which is the
##            picture that is changed to. 
pic.table <- function(header = NULL,pics = NULL,data = NULL,use.names = FALSE,suffix =".png",
                      width = 400,height = 400,blankname = NULL,mapnames = NULL,maps = NULL, links = NULL)
{
  ## create a blank image for the starter.
  if (is.null(blankname))
    {
      blankname <- "blank.png"
      get.blank(fname = blankname)
    }
  ## if there are maps, inlcude them.
  res <- ""
  if (!is.null(maps))
    {
      res <- paste(maps,collapse = "\n")
    }
  ## start the table 
  res <- paste(res,"\n<table border>",sep ="")
  ## Java-script needed for the picture change.
  res <- paste(res,"<SCRIPT LANGUAGE=\"JAVASCRIPT\">\nif (document.images) \n{",sep = "\n")
  for (i in 1:length(pics))
    {
      tmp1 <- paste("img",i,"=new Image();",sep ="")
      tmp2 <- paste("img",i,".src = \"",as.character(pics[i]),"\";",sep ="")
      res <- paste(res, tmp1,tmp2,sep ="\n")
    }
  res <- paste(res,"\n}\n")
  if (!is.null(maps))
    {
      tmpa <- NULL
      for (i in 1:length(maps))
        {
          tmp <- paste("strmap",i," = new String(\"#",mapnames[i],"\");",sep ="")
          tmpa <- c(tmpa,tmp)
        }
      res <- paste(res,paste(tmpa,collapse = "\n"))
      tmpa <- "\nfunction change(map,image)\n{\ndocument.imagename.src = image;\n"
      tmpa <- paste(tmpa,"document.imagename.useMap = map;\nreturn true;\n}\n",sep ="")
      res <- paste(res,tmpa)
    }else{
      tmpa <- "\nfunction change(image)\n{\ndocument.imagename.src = image;\n"
      tmpa <- paste(tmpa,"return true;\n}\n",sep ="")
      res <- paste(res,tmpa)
    }
  res <- paste(res,"</SCRIPT>\n",sep ="\n")
  ## the table headings
  nr.rows <- dim(header)[1]
  nr.cols <- dim(header)[2]
  nr.pics <- length(pics)

  for (rw in 1:nr.rows)
    {
      res <- paste(res,"<tr>")
      for (co in 1:nr.cols)
        {
          if (nchar(header[rw,co]) != 0)
            {
              tmp <- paste("<th>",header[rw,co],"</th>",sep = "")
              res <- paste(res,tmp)
            }else{
              res <- paste(res,"<td></td>")
            }
        }
      ## this is supposed to happen: An Extra column for the pictures.
      if (rw != nr.rows)
        {
          res <- paste(res,"<td></td></tr>\n")
        }else{
          res <- paste(res,"<td rowspan = \"",nr.pics+1,
                       "\"><img src = \"",blankname,"\" name=\"imagename\" usemap = \"\" width=\"",
                       width,"\" height=\"",height,"\" border=\"0\"</td></tr>\n",sep ="")
        }
    }
  
  ## the table itself
  nr.rows <- dim(data)[1]
  nr.cols <- dim(data)[2]
  if (use.names)
    {
      col.start = 2
    }else{
      col.start = 1
    }
  for (p in 1:nr.pics)
    {
      tmp <- "<tr><td>"
      name <- NULL
      if (use.names)
        {
          name <- paste(data[p,1],sep ="")
        }else{
          name <- paste(strsplit(pics[p],suffix)[[1]],sep = "")
        }
      ## what should be opened on 'click'
      if (is.null(links))
        {
          tmp <- paste(tmp,"<a href = \"",pics[p],"\" target = \"_blank\"",sep ="")
        }else{
          tmp <- paste(tmp,"<a href = \"",links[p],"\" target = \"_blank\"",sep ="")
        }
      ## what should happen on 'mouseOver'
      if (is.null(maps))
        {
          tmp <- paste(tmp," onMouseover=change(img",p,".src)>",name,"</a></td>",sep ="")
        }else{
          tmp <- paste(tmp," onMouseover=change(strmap",p,",img",p,".src)>",name,"</a></td>",sep ="")
        }
      if(is.numeric(nr.cols) && nr.cols >= col.start){
        for (i in col.start:nr.cols)
          {
            tmp <- paste(tmp,"<td>",data[p,i],"</td>",sep ="")
          }
      }

      res <- paste(res,tmp,"</tr>\n",sep = "")
      
  }
  ## ends the table.
  res <- paste(res,"</table>\n",sep ="\n")
}

## make.pic.link, takes an array of names(strings) and returns an array of strings where the
##                names are formatted to html <IMG>-tag formatted to size and linking to the image itself.
##
## Notes: -.
##
## Todo: -.
##
## Parameters:
## fnames:      (strings, array) Names of the pictures give.
## caps:        (strings, optional) Caption to be displayed when mouse is over image. Either as long as fnames
#                                  or shorter which will then be repeated.
## heigth:      (numeric) Heigth of the image.
## width:       (numeric) Width of the image.
## path:        (string) Where the images are. Remember the '/'
## thumbPrefix: (string,optional) If there are thumbnails of the pictures, set the suffix to the names here.
##

make.pic.link <- function(fnames,caps = "Open Larger Image", thumbPrefix = "", path = "",height = 50,width = 50)
{
  ## sanity check. 
  if (length(fnames) < 1)
    {return("")}
  if (length(caps) < length(fnames))
    {
      caps <- rep(caps,length(fnames)/length(caps))
    }
  res <- NULL
  for (i in 1:length(fnames))
    {
      picsrc<-paste(path,fnames[i],sep = "")
      thumbsrc <- paste(path,thumbPrefix,fnames[i],sep ="")
      tmp <- paste("<a href=\"",picsrc,"\"><img src=\"",thumbsrc,"\" width=\"",width,"\" height=\"",height,
                   "\" border=\"0\" title=\"",caps[i],"\"></a>",sep = "")
      res <- c(res,tmp)
    }
  return(res)
}

## make.HEAT.map, using the R-coordinates and wanted html-size, creates an imageMap that will display the given text on
## mouseover.
##
## Notes: -. 
##
## Todo: -.
##
## Parameters:
## texts:    (String,array) Texts to be displayed.
## nr.cols:  (numeric) Number of logical columns, if != 1 then texts must be a multiple of this.
## mapn:     (String) Name of the map created.
## hh:       (numeric) Heigth of image in imagemap
## hw:       (numeric) Width of image in imagemap
## links:    (string, optional) if given this is where a click on the map-instance takes you. 

make.HEAT.imagemap <- function(texts = "",start = 0,stp = 100 ,mapn = "map1",hh = 100,hw = 100, links = NULL)
  {
    res <- paste("<map name=\"",mapn,"\">",sep ="")
    if (length(texts) >= 1)
      {
        nr.rows <- length(texts)
        box.h <- hh/nr.rows
        ## loop, and create the map instances.
        for (i in 1:nr.rows)
          {
            newC <- c(start,(i-1)*box.h,stp,i*box.h)
            tmp <- paste("<area shape=rect coords=\"",newC[1],",",newC[2],",",newC[3],",",newC[4],"\"",sep ="")
            if (!is.null(links))
              {
                tmp <- paste(tmp," href = \"",links[i],"\"  target = \"_blank\" ",sep ="")
              }
            tmp <- paste(tmp," title = \"",texts[i],"\">",sep = "")
            res <- c(res,tmp)
          }
      }
    res <- c(res,"<area shape=default nohref>","</map>")
    return(paste(res,collapse = "\n"))
  }


## make.GO.map, using the R-coordinates and wanted html-size, creates an imageMap that will display the given text on
## mouseover.
##
## Notes: -.
##
## Todo: -.
##
## Parameters:
## grph:     (Rgraph) An Rgraph-object defining the layout of the current graph. (returned by generateGOgraph (GOfunc.R))
## texts:    (String,array) Same length (or repeted) as number of nodes with texts to be displayed.
## mapn:     (String) Name of the map created.
## hh:       (numeric) Heigth of image in imagemap
## hw:       (numeric) Width of image in imagemap
## offset:   (numeric,array) Extremes of the plot(as given by par("usr"))
## pvals:    (numeric,list,optional) GO-indexed list with pvalues to be displayed along with the GOid.
## links:    (String,optional) If specified, clicking inside a box will take you here,
##                             if not to the GO-browser at ebi,should countain a default called "default"
## defLink:  (String,optional) If specified, clicking outside specified coordinates will take you here (URL,link). 

make.GO.map <- function(grph,pvals = NULL,texts = "",mapn = "map1",hh = 100,hw = 100,offset = c(0,0,0,0),
                        links = NULL,defLink = NULL)
  {
    res <- paste("<map name=\"",mapn,"\">",sep ="")
    ##print("make.GO.graph")
    ##print(length(grph@AgNode))
    if (length(grph@AgNode) >= 1)
      {
        ## make sure there are texts enough
        if (length(grph@AgNode) < length(texts))
          {
            texts <- rep(texts,length(grph@AgNode)/length(texts))
          }
        
        ## define extreme size of plot.
        plotxmin <- min(grph@boundBox@botLeft@x,offset[1])
        plotymin <- min(grph@boundBox@botLeft@y,offset[3])
        plotxmax <- max(grph@boundBox@upRight@x,offset[2])
        plotymax <- max(grph@boundBox@upRight@y,offset[4])
        
        ## create transformation function for p-coords -> h-coords
        plotSizeX <- plotxmax - plotxmin
        plotSizeY <- plotymax - plotymin
        print(paste(plotSizeX,plotSizeY))
        
        xscale <- hw/plotSizeX
        yscale <- hh/plotSizeY
        
        ## make a local function that moves a box' coordinates (upleft,lowright).
        newCo <- function(oC)
          {
            ## "hh-" because R and html have their origo on different locations.
            nul1 <- (oC[1] - plotxmin)*xscale
            nul2 <- hh-(oC[2] - plotymin)*yscale
            
            nlr1 <- (oC[3] - plotxmin)*xscale
            nlr2 <- hh-(oC[4] - plotymin)*yscale
            return(c(nul1,nul2,nlr1,nlr2))
          }
        ## loop over nodes and create the map instances.
        nodes <- grph@AgNode
        for (i in 1:length(nodes))
          {
            oldC <- c(nodes[[i]]@center@x - nodes[[i]]@lWidth, ## upleft x-coord
                      nodes[[i]]@center@y + nodes[[i]]@height/2, ## upleft y-coord
                      nodes[[i]]@center@x + nodes[[i]]@rWidth, ##  lowright x-coord
                      nodes[[i]]@center@y - nodes[[i]]@height/2) ## lowright y-coord
            addtxt <- ""
            if (!is.null(pvals))
              {
                ptmp <- NULL
                try(ptmp <- pvals[[nodes[[i]]@name]],silent = TRUE)
                print(paste(nodes[[i]]@name,"=>",ptmp))
                if (!is.null(ptmp))
                  {
                    if (!is.na(as.numeric(ptmp)))
                      {
                        addtxt <- paste("[",as.character(signif(as.numeric(ptmp),digits = 4)),"]",sep = "")
                      }else{
                        addtxt <- paste("[",ptmp,"]",sep ="")
                      }
                    print(paste("mapGO:",ptmp,addtxt))
                  }
              }
            newC <- newCo(oldC)
            if (is.null(links))
              {
                link <- paste("http://www.ebi.ac.uk/ego/DisplayGoTerm?id=",strsplit(texts[i]," ")[[1]][1],sep ="")
              }else{
                ltmp <- NULL
                try(ltmp <- links[[nodes[[i]]@name]],silent = TRUE)
                if (!is.null(ltmp))
                  {
                    link <- ltmp
                  }else{
                    ## wtf???
                    link <- links[["default"]]
                  }
              }
            ##print(paste(nodes[[i]]@name,"=>",link))
            res<-c(res,paste("<area shape=rect coords=\"",newC[1],",",newC[2],",",
                             newC[3],",",newC[4],"\" href = \"",link,
                             "\" target = \"sub_browser\" title = \"",paste(texts[i],addtxt,sep=" "),"\">",sep = ""))
          }
      }
    ##print(defLink)
    if (is.null(defLink))
      {
        res <- c(res,"<area shape=default nohref>","</map>")
      }else{
        ##res <- c(res,paste("<area shape=default href = \"",defLink,"\" target = \"_blank\" title = \"Open Larger Image (Huge)\">",
        ##sep =""),"</map>")
        res <- c(res,paste("<area shape=rect coords = \"0,0,",hh,",",hw,"\" href = \"",defLink,
                           "\" target = \"_blank\" title = \"Open Larger Image (Huge)\">",sep =""))
        res <- c(res,"<area shape=default nohref>","</map>")
      }
    return(paste(res,collapse = "\n"))
  }

##
## makeGO.<name>, a whole series of functions constructing different types of tables for viewing GO-related results.
##
## Notes: Most of these functions require data-structures to have been loaded by the calling plugin (SEGOTERM from SEGO.rda etc)
##
## Todo: -.
##


## makeGO.ptable, constructs a table with either over or under represented GO-terms from the given structures.
##
## Notes: Requires SEGOTERM
##
## Todo: -.
##
## Parameters:
## significant.terms:   (structure) Contains pvalues for GOids in s.t[[groups]][[direction]][[gid]]-structure
## groups:              (numeric array,list) Which Assays in the MA-experiment are supposed to be viewed together
## direction:           (string,"up"or "down") Are we looking for over or unde representation
## title:               (string) Title for the table.
## col:                 (string, optional) Color for the pvalues in the table. (red-over, blue - under)
## co:                  (numeric) Cutoff for significanse (two-sided p-values)
## gInfo:               (boolean,optional) If pvalues should link to a geneInfo file. 
## ON:                  (string,optional) if gInfo is used this need to be set. 
## upimg:               (character) if given, a path to a (very) small image with and up-arrow
## dwimg:               (character) like upimg, but down.
makeGO.ptable <- function(significant.terms, groups, direction = "up", title = "" , col = NULL,co = 0.05,gInfo = FALSE, ON = NULL)
{
  cols <- list()
  cols[["up"]] <- "red"
  cols[["down"]] <- "blue"
  if (is.null(col))
    {
      col <- cols[[direction]]
    }
  ##
  ## table and header
  res <- "<table border>"
  nr.groups <- length(groups)
  res <- c(res,paste("<tr><th colspan = \"",nr.groups+1,"\" align = \"center\">",title,"</th></tr>",sep = ""))
  res <- c(res,"<tr><td>GO id,term</td>")
  ##
  ## the row with the group names
  all <- NULL
  tmp <- NULL
  for (i in 1:nr.groups)
    {
      all <- union(all,dimnames(as.array(significant.terms[[i]][[direction]]))[[1]])
      tmp <- c(tmp, paste("<td>",paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),"</td>",sep = ""))
    }
  tmp <- c(tmp,"</tr>")
  res <- c(res, paste(tmp,collapse = ""))
  ##
  ## the data
  if (length(all)>0)
    {
      for (i in 1:length(all))
        {
          ## a row
          tmpr <- paste("<tr><td>",all[i]," ",get(all[i],SEGOTERM),"</td>",sep = "")
          for (j in 1:length(groups))
            {
              ## those that make the cutoff gets color
              if (significant.terms[[j]]$all[[all[i]]]<co)
                {
                  if (gInfo)
                    {
                      link <- paste("<a href = \"geneInfo",ON,paste("[",paste(groups[[j]],collapse = ","),"]",sep = "")
                                    ,".html#",all[i],"\" target = \"sub_browser\"> <font color = \"",col,"\">",
                                    significant.terms[[j]]$all[[all[i]]],"</font></a>",sep ="")
                      tmpr <- c(tmpr,paste("<td>",link,"</td>",sep = ""))                          
                    }else{
                      tmpr <- c(tmpr,paste("<td><font color = \"",col,"\">",significant.terms[[j]]$all[[all[i]]],
                                           "</font></td>",sep =""))
                    }
                }else{
                  tmpr <- c(tmpr,paste("<td>",significant.terms[[j]]$all[[all[i]]],"</td>",sep =""))
                }
            }
          tmpr <- c(tmpr,"</tr>")
          res <- c(res,paste(tmpr,collapse = ""))
        }
    }else{
      res <- c(res,paste("<tr><td colspan =\"",length(groups) +1
                         ,"\" align = \"center\">No significant results.</td></tr>",sep = ""))
    }
  ##
  ## and finally
  res <- c(res,"</table>")
  return(paste(res,collapse <- "\n"))
}

makeGO.ptable_V2<- function(significant.terms, groups, direction = "up", title = "" , col = NULL,co = 0.05,gInfo = FALSE, ON = NULL,upimg = NULL,dwimg = NULL)
{
  cols <- list()
  cols[["up"]] <- "red"
  cols[["down"]] <- "blue"
  if (is.null(col))
    {
      col <- cols[[direction]]
    }
  ##
  ## table and header
  res <- "<table border>"
  nr.groups <- length(groups)
  res <- c(res,paste("<tr><th colspan = \"",nr.groups+1,"\" align = \"center\">",title,"</th></tr>",sep = ""))
  res <- c(res,"<tr><td>GO id,term</td>")
  ##
  ## the row with the group names
  all <- NULL
  tmp <- NULL
  for (i in 1:nr.groups)
    {
      all <- union(all,names(significant.terms[[i]][[direction]]))
      tmp <- c(tmp, paste("<td>",paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),"</td>",sep = ""))
    }

  go.order <- array(NA,0)
  for(i in 1:length(all))
    {
      min <- 1
      for(j in 1:nr.groups)
        {
          ## one will be existing since the term is in all
          try(if (significant.terms[[j]][[direction]][all[i]] < min)
              {
                min <- significant.terms[[j]][[direction]][all[i]]
                go.order[all[i]] <- min
              },silent = TRUE)
        }
    }
  go.order <- sort(go.order)
  go.order <- names(go.order)
  
  tmp <- c(tmp,"</tr>")
  res <- c(res, paste(tmp,collapse = ""))
  ##
  ## the data
  all  <- go.order
  SKIP <- FALSE
  if (length(all)>0)
    {
      for (i in 1:length(all))
        {
          ## a row
          tmpr <- paste("<tr><td>",all[i]," ",get(all[i],SEGOTERM),"</td>",sep = "")
          for (j in 1:length(groups))
            {
              ## those that make the cutoff gets color

              ## existing in this group?
              if (!is.na(significant.terms[[j]]$all[all[i]]))
                {
                  ## significant?
                  if (!is.na(significant.terms[[j]][[direction]][all[i]]))
                    {
                      ## shall we link to extra info?
                      if (gInfo)
                        {
                          link <- paste("<a href = \"geneInfo",ON,paste("[",paste(groups[[j]],collapse = ","),"]",sep = "")
                                        ,".html#",all[i],"\" target = \"sub_browser\"> <font color = \"",col,"\">",
                                        abs(significant.terms[[j]]$all[all[i]]),"</font></a>",sep ="")
                          tmpr <- c(tmpr,paste("<td>",link,"</td>",sep = ""))                          
                        }else{
                          tmpr <- c(tmpr,paste("<td><font color = \"",col,"\">",abs(significant.terms[[j]]$all[all[i]]),
                                           "</font></td>",sep =""))
                        }
                    }else{
                      if(sign(significant.terms[[j]]$all[all[i]]) == -1)
                        {
                          if(!is.null(dwimg))
                            {
                              tmp.pic <- paste("<img src=\"",dwimg,"\">",sep = "")
                            }else{
                              tmp.pic <- ""
                            }
                        }else{
                          if(!is.null(upimg))
                            {
                              tmp.pic <- paste("<img src=\"",upimg,"\">",sep = "")
                            }else{
                              tmp.pic <- ""
                            }
                        }
                      tmpr <- c(tmpr,paste("<td>",tmp.pic,abs(significant.terms[[j]]$all[all[i]]),"</td>",sep =""))
                    }
                }else{
                  ## this happens when the background for a time point is unannotated. 
                  tmpr <-  c(tmpr,paste("<td>*</td>",sep =""))
                  SKIP <- TRUE
                }
            }
          tmpr <- c(tmpr,"</tr>")
          res <- c(res,paste(tmpr,collapse = ""))
        }
    }else{
      res <- c(res,paste("<tr><td colspan =\"",length(groups) +3
                         ,"\" align = \"center\">No significant results.</td></tr>",sep = ""))
    }
  ##
  ## and finally
  res <- c(res,"</table>")
  if(SKIP)
    res <- c(res,"<br><b>* These terms were not used in the background for this timepoint. The p-values cannot be calculated.</b><br>")
  return(paste(res,collapse <- "\n"))
}


## makeGO.strucuredtable, draws the induced GO-structure in a table where indentation reflects depth. 
##
## Notes: Requires SEGOTERM,SEGOCHILDREN & "GOfunc.R"
##
## Todo: -.
##
## Parameters:
## significant.terms:   (structure) Contains pvalues for GOids in s.t[[groups]][[direction]][[gid]]-structure
## ON:                  (string, "CC","MF" or "BP") Which Ontology to use.
## groups:              (numeric array,list) Which Assays in the MA-experiment are supposed to be viewed together
## co:                  (numeric) Cutoff for significanse (two-sided p-values)
## gInfo:            (boolean,optional) if true, every GOid links to a geneInfo-page.
## upimg:               (character) if given, a path to a (very) small image with and up-arrow
## dwimg:               (character) like upimg, but down.

makeGO.structuredtable_V2<- function(significant.terms,ON,groups,gInfo = FALSE,upimg = NULL,dwimg = NULL)
{
  ## helper, print recursively down the GO-structure.
  ## uses 'significant.terms' from the Main-function space.
  ## 'original' are all up or down all groups
  leveller <- function(node,original,extended,space)
    {
      ## get this row done.
      res <- paste("<tr><td>",paste(space,node,sep = "")," ",get(node,SEGOTERM),"</td>",sep = "")   
      ## colors in the table depend on whether we're induced or not.
      for (j in 1:length(groups))
        {
          ## is this GO-term in our group ?
          if(!is.na(significant.terms[[j]]$all[node]))
            {
              col.tmp <- "black"
              pic.tmp <- ""
              if (sign(significant.terms[[j]]$all[node]) == -1 && !is.null(dwimg))
                {
                  pic.tmp <- paste("<img src=",dwimg,">   ",sep ="")
                }
               if (sign(significant.terms[[j]]$all[node]) == 1 && !is.null(upimg))
                {
                  pic.tmp <- paste("<img src=",upimg,">   ",sep ="")
                }
              if(!is.na(significant.terms[[j]]$up[node]))
                {
                  col.tmp <- "red"
                  pic.tmp <- ""
                }
              if(!is.na(significant.terms[[j]]$down[node]))
                {
                  col.tmp <- "blue"
                  pic.tmp <- ""
                }
              ## if the node is significant have it link to the geneInfo, if wanted (gInfo)
              if (gInfo)
                {
                  link <- paste(pic.tmp,"<a href = \"geneInfo",ON,paste("[",paste(groups[[j]],collapse = ","),"]",sep = "")
                                ,".html#",node,"\" target = \"sub_browser\"> <font color = \"",col.tmp,"\">",
                                abs(significant.terms[[j]]$all[node]),"</font></a>",sep ="")
                  res <- paste(res,"<td>",link,"</td>",sep = "")            
                }else{
                  res <- paste(res,"<td>",pic.tmp,"<font color = \"",col.tmp,"\">",abs(significant.terms[[j]]$all[node]),
                               "</font></td>",sep ="")
                }
            }else{ ## not in our group at all.
              res <- paste(res,"<td>-</td>",sep ="")
            }
        }
      res <- paste(res,"</tr>\n",sep = "")

      ## prepare for the recursive call
      space <- paste(space,"....",sep ="")
      level.kids <- NULL
      try(level.kids <- get(node,SEGOCHILDREN),silent = TRUE)
      ## kids within the family
      internal.level.kids <- intersect(level.kids,extended)
      if (length(internal.level.kids)>0)
        {
          for(i in 1:length(internal.level.kids))
            {
              res <- paste(res,leveller(internal.level.kids[i],original,extended,space))
            }
        }else{
          return(res)
        }
    }
  ##
  ## MAIN:
  ##
  
  res <- "<table border>"
  nr.groups <- length(groups)
  ## headers & construnction of leaves.
  res <- c(res,paste("<tr><th colspan = \"",nr.groups+1,"\" align = center>Two sided pvalues</th></tr>",sep = ""))
  tmpr <- "<tr><td>GO id,term</td>"
  all.sig <- NULL ## will contain all 'leaves'
  for (i in 1:nr.groups)
    {
      all.sig <- union(all.sig,names(significant.terms[[i]]$up))
      all.sig <- union(all.sig,names(significant.terms[[i]]$down))
      tmpr <- c(tmpr,"<td>",paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),"</td>")
    }
  tmpr <- c(tmpr,"</tr>\n")
  res <- c(res,paste(tmpr,collapse = ""))

  ## get all induced nodes and call the 'leveller'
  if (length(all.sig)>0)
    {
      ##print("continuing")
      topn <- topnode[[ON]]
      sparseTree <- getSparseTree(all.sig)
      res <- c(res,leveller(topn,all.sig,sparseTree,""))
      res <- c(res,"</table>")
    }else{
      res <- c(res,paste("<tr><td colspan =\"",length(groups) +1,
                         "\" align = \"center\">No significant results.</td></tr></table>",sep = ""))
    }
}




##  makeGOpage, builds a list of GO->gene mappings, has anchors in it for easy managment. 
## 
## Notes: makeGOpage_V2, works with 'env' Version 2, othervise all arguments are the same
##                       except that idfilter shouldn't be needed.
##
## Todo: -.
##
## Parameters:
## env:      (environment) All info about the GO->gene mappings in the current background.
## gids:     (String,array) Which GOids that are to be displayed.
## defLink:  (String,optional) if given, an empty entry named this will be created. (used for missing links etc)
## noCnt:    (boolean, optional) if set to True, no multiplicity will be displayed after the reporter id.
## idfilter: (array,optional) repIds (or how the system might be setup) that are to be displayed. 

makeGOpage <- function(env,gids,defLink = NULL,noCnt = FALSE, idfilter = NULL)
  {
    res <- NULL
    ## these should all exists since gids are a subset of the background. 
    ## if there are no significants hits whatsoever, gids = NULL. 
    if (!is.null(gids))
      {
        allInfo <- mget(gids,env,ifnotfound = NA)
        nr.g <- length(allInfo)
        for(i in 1:nr.g)
          {
            gid <- gids[i]
            rows <- NULL
            try(rows <- allInfo[[gid]],silent = TRUE)
            if (!is.na(rows[[1]]))
              {
                ## headers and such.
                res <- c(res,paste("<a name=\"",gid,"\">",sep =""))
                res <- c(res,"<table align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">")
                gidLink <- paste("<a target = \"_blank\" href = http://www.ebi.ac.uk/ego/DisplayGoTerm?id=",gid,">",gid,"</a>",sep ="")
                res <- c(res,paste("<tr><th colspan = \"3\" align = \"left\">",gidLink,"</th></tr>",sep =""))
                res <- c(res,"<tr><th class=\"subhead\">Reporter</th><th class=\"subhead\">Locus Link</th><th class=\"subhead\">Gene Name</th></tr>")
                counts <- array(NA,0)  ## rows[[]][1]
                infoOne <- array(NA,0) ## rows[[]][2]
                infoTwo <- array(NA,0) ## rows[[]][3]
                ## re-organize the information so that we rid ourselves of duplicates(keep track of how many duplicates)
                for (r in 1:length(rows))
                  {
                    ## tmp is one entry, form c(id,LL,text)
                    tmp <- counts[rows[[r]][1]]
                    ## should this id be displayed ? 
                    if (!is.null(idfilter))
                      {
                        if( length(intersect(as.character(rows[[r]][1]),idfilter)) > 0)
                          {
                            ## this works becauase arrays don't use partial matching for character-indecies.
                            if (is.na(tmp[[1]]))
                              {
                                counts[rows[[r]][1]] <- 1
                                infoOne[rows[[r]][1]] <- rows[[r]][2]
                                infoTwo[rows[[r]][1]] <- rows[[r]][3]
                              }else{
                                counts[rows[[r]][1]] <- counts[rows[[r]][1]] +1
                              }
                          }
                      }else{
                        if (is.na(tmp[[1]]))
                          {
                            counts[rows[[r]][1]] <- 1
                            infoOne[rows[[r]][1]] <- rows[[r]][2]
                            infoTwo[rows[[r]][1]] <- rows[[r]][3]
                          }else{
                            counts[rows[[r]][1]] <- counts[rows[[r]][1]] +1
                          }
                      }
                  }
                
                repOrder <- order(as.numeric(dimnames(as.array(counts))[[1]]))
                counts <- counts[repOrder]
                infoOne <- infoOne[repOrder]
                infoTwo <- infoTwo[repOrder]
                
                for (r in 1:length(counts))
                  {
                    repid <- dimnames(as.array(counts[r]))[[1]]
                    if (noCnt)
                      {
                        cntStr <- ""
                      }else{
                        cntStr <- paste(" (",counts[r],")",sep ="")
                      }
                    rowLine <- paste("<tr><td>",
                                     repid,cntStr,
                                     "</td><td>",
                                     infoOne[repid],
                                     "</td><td>",
                                     infoTwo[repid],"</td></tr>",sep ="")
                    res <- c(res, rowLine) 
                  }
                res <- c(res,"</table></a><br>")
              }else{
                print(paste(gid,"not found"))
              }
          }
      }
    if (!is.null(defLink))
      {
        ## headers and such.
        res <- c(res,paste("<a name=\"",defLink,"\">",sep =""))
        res <- c(res,"<table align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">")
        res <- c(res,paste("<tr><th colspan = \"3\" align = \"left\">",defLink,"</th></tr>",sep =""))
        rowLine <- paste("<tr><td colspan = \"3\" align = \"left\">","No Information Found.",sep ="")
        res <- c(res, rowLine)
        res <- c(res,"</table></a><br>")
      }
    return(paste(res,sep ="\n"))
  }


## just a fixed version, should probably rewrite from scratch. 
## t1-t3: header texts, in V2 you can choose, headers should change
makeGOpage_V2 <- function(env,gids,defLink = NULL,noCnt = FALSE, idfilter = NULL,
                          t1 = "Locus Link", t2 = "Reporter", t3 = "Gene Name",bgEnv = NULL,group = NULL)
  {
    countRows <- function(my.rows,r)
      {
        counts <- array(NA,0)  ## rows[[]][1]
        infoOne <- array(NA,0) ## rows[[]][2]
        infoTwo <- array(NA,0) ## rows[[]][3]
        ## re-organize the information so that we rid ourselves of duplicates(keep track of how many duplicates)
        ##for (r in 1:length(my.rows)) ## this is already split on the groups we want.
        ##  {
        all.maps <- NULL
        all.ids  <- NULL
        all.nms  <- NULL
        
        for(i in r)
          {
            try(all.maps <- c(all.maps,unlist(my.rows[[i]][,1])),silent =TRUE)
            try(all.ids  <- c(all.ids,unlist(my.rows[[i]][,2])),silent =TRUE)
            try(all.nms  <- c(all.nms,unlist(my.rows[[i]][,3])),silent =TRUE)
          }
        if(!is.null(all.ids))
          {
            ids <- unique(all.ids)
            for (j in ids)
              {
                if (is.null(idfilter) || (length(intersect(idfilter,j)) > 0))
                  {
                    ## seen before ? 
                    if (is.na(counts[j]))
                      {
                        pos <- which(all.ids == j)
                        counts[j]  <- length(pos)
                        infoOne[j] <- all.maps[pos[1]]
                        infoTwo[j] <- all.nms[pos[1]]
                      }
                  }
              }
          }
        ##  }
        ## right! order! return!
        repOrder <- order(as.numeric(names(counts)))
        counts   <- counts[repOrder]
        infoOne  <- infoOne[repOrder]
        infoTwo  <- infoTwo[repOrder]
        return(list(repOrder,counts,infoOne,infoTwo))
      }
    
    
    res <- NULL
    ## these should all exists since gids are a subset of the background. 
    ## if there are no significants hits whatsoever, gids = NULL. 
    if (!is.null(gids))
      {
        allInfo <- mget(gids,env,ifnotfound = NA)
        if(!is.null(bgEnv))
          {
            allBGInfo <- mget(gids,bgEnv,ifnotfound = NA)
          }
        nr.g <- length(allInfo)
        for(i in 1:nr.g)
          {
            gid <- gids[i]
            rows <- NULL
            try(rows <- allInfo[[gid]],silent = TRUE)
            bgRows <- NULL
            try(bgRows <- allBGInfo[[gid]],silent = TRUE)
            
            if (!is.na(rows))
              {
                ## extract info from FG
                tmp <- countRows(rows,group)
                repOrder <- tmp[[1]]
                counts   <- tmp[[2]]
                infoOne  <- tmp[[3]]
                infoTwo  <- tmp[[4]]
                ##print("FG:")
                ##print(counts)
                nrAFG    <- sum(counts)
                
                ## extract info from BG
                tmp   <- countRows(bgRows,group)
                ##print("BG:")
                ##print(tmp[[2]])
                nrABG <- sum(tmp[[2]]) 
                
                ## headers and such.
                res <- c(res,paste("<a name=\"",gid,"\">",sep =""))
                res <- c(res,"<table align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">")
                gidLink <- paste("<a target = \"_blank\" href = http://www.ebi.ac.uk/ego/DisplayGoTerm?id=",
                                 gid,">",gid,"</a>   ",nrAFG," (",nrABG,") Annotations.",sep ="")
                res <- c(res,paste("<tr><th colspan = \"3\" align = \"left\">",gidLink,"</th></tr>",sep =""))
                ## although the info is given in t1-t3 order, t2-t1-t3 is more natural to display
                res <- c(res,"<tr><th class=\"subhead\">",t2,"</th><th class=\"subhead\">",t1,"</th>",
                         "<th class=\"subhead\">",t3,"</th></tr>")
                
                for (r in 1:length(counts))
                  {
                    repid <- names(counts[r])
                    if (noCnt)
                      {
                        cntStr <- ""
                      }else{
                        cntStr <- paste(" (",counts[r],")",sep ="")
                      }
                    rowLine <- paste("<tr><td>",
                                     repid,cntStr,
                                     "</td><td>",
                                     infoOne[repid],
                                     "</td><td>",
                                     infoTwo[repid],"</td></tr>",sep ="")
                    res <- c(res, rowLine) 
                  }
                res <- c(res,"</table></a><br>\n")
              }else{
                print(paste(gid,"not found"))
              }
          }
      }
    if (!is.null(defLink))
      {
        ## headers and such.
        res <- c(res,paste("<a name=\"",defLink,"\">",sep =""))
        res <- c(res,"<table align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">")
        res <- c(res,paste("<tr><th colspan = \"3\" align = \"left\">",defLink,"</th></tr>",sep =""))
        rowLine <- paste("<tr><td colspan = \"3\" align = \"left\">","No Information Found.",sep ="")
        res <- c(res, rowLine)
        res <- c(res,"</table></a><br>\n")
      }
    return(paste(res,sep ="\n"))
  }

## select.form, creates a HTML form including a select structure
##
## Notes: To be used together with contents.div
##        Requires that 'content' and 'selectedcontent' are defined in the
##        <style> section of the HTML file. Also requires that the java script
##        defining switchOn is included in the HTML.
##
## Todo:  
##
## Parameters:
## select.options     (string,optional)  Names of select options. Can be NULL
##                    if use.names == FALSE
## no.select.options  (integer,optional) The number of select options, if
##                    use.names == TRUE, no.select.options is set to the length
##                    of select.options
## use.names          (boolean,optional) TRUE if the select options should be named
##                    as defined in select.options
## form.name          (string,optional)  String defining the form name
## form.no            (integer,optional) Integer extending the form name if more than
##                    one form is to be used
##
select.form <- function(select.options = NULL, no.select.options = 0, use.names=FALSE,
                        form.name = "selectform", form.no = 1){

  ##Identify number of options in the select form
  no.items <- length(select.options)
  ##Set the form name
  form.name <- paste(form.name,form.no,sep="")
  ##Set the zero option (none selected)
  zero.option <- "-"
  if (use.names) {
    zero.option <- "None"
    no.select.options <- no.items
  }
  else
    select.options <- 1:no.select.options

  contents.id <- paste("contents",form.no,sep="")
  res <- paste("<FORM NAME=",form.name,">\n",sep="")
  res <- paste(res,"<SELECT NAME=sel onChange=\"switchOn(document.",form.name,".sel.value,'",
               contents.id,"');\">\n",sep="")
  options <- paste("<OPTION VALUE=0>",zero.option,"\n",sep="")

  ##Generate options
  if (no.select.options > 0) {
    for (i in 1:no.select.options)
      options <- paste(options,"<OPTION VALUE=",i,">",select.options[i],"\n",sep="")
  }
  res <- paste(res,options,"</SELECT></FORM>\n")
  return(res)
}

## contents.div, Creates a div block with contents that are displayed by the switchOn
##               java script function according to a select form generated by the
##               select.form function 
##
## Notes: To be used together with select.form
##        Requires that 'content' and 'selectedcontent' are defined in the
##        <style> section of the HTML file. Also requires that the java script
##        defining switchOn is included in the HTML.
##
## Todo:  
##
## Parameters:
## contents.data      (list of vectors of strings)  List of vectors of picnames
##                    including path. Specifies the images to be displayed in the
##                    content tables
## contents.links     (list of vectors of strings,optional)  List of vectors of picnames
##                    including path. Specifies the images to be linked by the images
##                    specified in contents.data. Can be NULL.
## use.links          (boolean,optional) TRUE the contents.data images should
##                    link to the contents.links (if contents.links != NULL)
##                    or the contents.data (if contents.links == NULL)
## contents.no        (integer,optional) Integer extending the div id if more than
##                    one form is used
##
contents.div <- function(contents.data = NULL, contents.links = NULL,
                         use.links = TRUE, no.cols = 4, contents.no = 1){

  contents.id <- paste("contents",contents.no,sep="")
  res <- paste("<DIV ID=\"",contents.id,"\">\n",sep="")
  contents <- ""

  no.contents <- length(contents.data)
  no.links <- length(contents.links)
  ##Should contents.data link to contents.data?
  if (use.links && (no.links != no.contents)) {
    no.links <- no.contents
    contents.links <- content.data
  }
  
  if (no.contents > 0) {
    for (i in 1:no.contents){
      content <- paste("<DIV CLASS=\"content\" ID=\"",contents.id,"_",i,"\">\n",sep="")
      ##Generate a table of images defined in the vectors of each list item
      content <- paste(content,"<TABLE>\n<TR>\n",sep="")
      
      data <- contents.data[[i]]
      links <- contents.links[[i]]
      no.items <- length(data)
      if (no.items > 0) {
        ##Identify the table layout
        no.rows <- 1
        if (no.items < no.cols)
          no.cols <- no.items
        else
          no.rows <- ceiling(no.items / no.cols)
        
        no.cells <- no.rows * no.cols
        counter <- 1

        ##Generate the contents...
        for (j in 1:no.items) {
          if (counter > no.cols) {
            content <- paste(content, "</TR>\n<TR>\n",sep="")
            counter <- 1
          }
          content <- paste(content,"<TD>\n",sep="")
          ##Should the images link to other images?
          if (use.links)
            content <- paste(content, "<A HREF=\"",links[j],"\"",sep="")
          ##Include the image
          content <- paste(content,"<IMG SRC=\"",data[j],"\">",sep="")
          if (use.links)
            content <- paste(content, "</A>",sep="")
          content <- paste(content,"\n</TD>\n",sep="")
          counter <- counter + 1
        }
        ##Fill in with TDs if necessary
        if (no.cells > no.items){
          for (k in 1:(no.cells - no.items))
            content <- paste(content, "<TD>\n</TD>\n",sep="")
        }
      }
      content <- paste(content,"</TR>\n</TABLE></DIV>\n",sep="")
      contents <- paste(contents,content,sep="")
    }
  }
  res <- paste(res,contents,"</DIV>\n",sep="")
  return(res)
}

## get.style, Returns a <style> block string
##
## Notes: 
##
## Todo:  
##
## Parameters:
## include.head   (boolean,optional) Should the <head> tag be included?
## include.body   (boolean,optional) Should the init <body> tag be included?
## include.content.java.script (boolean,optional) Should the java script for contents.div
##                             be included?
## add.to.style   (string,optional)  Extra style settings to be included
##
get.style <- function(include.head = TRUE, include.body = TRUE, include.style.tags = TRUE, include.content.java.script = FALSE, add.to.style="", add.to.script="") {

  style.string <- " .uname { color: #005040 }\n .iname { color: #500050 }\n input.tiny { font-size: 8pt }\n .version { font-size: 8pt; font-style: italic; color: gray; text-align: center; font-family: Helvetica; }\n a:active { text-decoration: none; color: #ff5050; }\n a:link { text-decoration: none; color: #1000d0; }\n a:visited { text-decoration: none; color: #1000d0; }\n a.ext:active { text-decoration: none; color: #ff8050; }\n a.ext:link { text-decoration: none; color: #f05000; }\n a.ext:visited { text-decoration: none; color: #800000; }\n body { font-family: Helvetica, sans-serif; background-color: #F8F8E0;font-size: 10pt;font-weight: normal }\n div { font-family: Helvetica, sans-serif; background-color: #F8F8E0;font-size: 10pt;font-weight: normal }\ntd { font-family: Helvetica, sans-serif; font-size: 10pt }\nth { font-family: Helvetica, sans-serif;  text-align: left;background-color: #D3D3BC; font-size: 10pt; }\ninput { font-family: Helvetica, sans-serif; font-size: 10pt }\nselect { font-family: Helvetica; font-size: 10pt }\nselect.mono { font-family: monospace; font-size: 10pt }\n .normal { font-size: 10pt;font-weight: normal }\n .large { font-size: 11pt }\n .help { font-style: italic }\n .special { font-weight: bold }\n .pagehead { text-align: left; background-color: #F8F8E0; font-size: 13pt }\n .subhead { text-align: left; font-size: 11pt; font-weight: bold }\n .subheadact { text-align: left; font-size: 11pt;font-weight: bold; background-color: #ffe0c0; }\n .red { color: Red; font-size: 11pt }\n .del { color: #e00050; font-weight: bold }\n a.popup { color: #007010 }\n .tiny { font-size: 8pt }\n .pink { background-color: #ffe0c0 }\n .content {display:none}\n .selectedcontent {display:block}\n "
    
  res <- ""
  if (include.head)
    res <- get.head()
  res <- paste(res,style.string,add.to.style,sep="")
  if (include.style.tags)
    res <- paste("<style type='text/css'>\n",res,"</style>\n",sep="")
  if (include.content.java.script)
    res <- paste(res,get.content.java.script(add.to.script=add.to.script),"\n",sep="")
  else if (add.to.script != "")
    res <- paste(res,"<script type=\"text/javascript\">\n",
                 add.to.script,"\n</script>",sep="")
  if (include.head) {
    res <- paste(res,"</head>\n",sep="")
    if (include.body)
      res <- paste(res,get.body(),"\n",sep="")
  }

  return(res)
}

##Helper function for get.style, could be used alone
get.head <- function(){
  return("<head>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">\n")
}

##Helper function for get.style, could be used alone
##bg.color could be set by the caller
get.body <- function(bg.color = "#F8F8E0"){
  return(paste("<body bgcolor=\"",bg.color,"\" text=black>",sep=""))
}

##Helper function for get.style, could be used alone
##Needed if contents.div and select.form is used
##add.to.script could be used if the caller wants to include extra java script functions
get.content.java.script <- function(add.to.script="") {
  return(paste("<script type=\"text/javascript\"> \n function switchOn(num,contentsid) { \n clearContents(contentsid); \n if (num != 0) {\n document.getElementById(contentsid+'_'+num).className = 'selectedcontent'; \n } \n } \n function clearContents(contentsid) { \n contents = document.getElementById(contentsid); \n if (contents != null) { \n contentsArray = contents.childNodes; \n for (var j=0; j<contentsArray.length; j++) { \n contentsArray[j].className = 'content'; \n } \n } \n } \n",add.to.script,"\n</script>",sep=""))
}

## makeGO.expandtable, draws the induced GO-structure in a expandable table
##
## Notes: Requires SEGOTERM,SEGOCHILDREN & "GOfunc.R"
##
## Todo: -.
##
## Parameters:
## significant.terms:   (structure) Contains pvalues for GOids in s.t[[groups]][[direction]][[gid]]-structure
## ON:                  (string, "CC","MF" or "BP") Which Ontology to use.
## groups:              (numeric array,list) Which Assays in the MA-experiment are supposed to be viewed together
## co:                  (numeric) Cutoff for significanse (two-sided p-values)
## gInfo:               (boolean,optional) if true, every GOid links to a geneInfo-page.
## upimg:               (character) if given, a path to a (very) small image with and up-arrow
## dwimg:               (character) like upimg, but down.
## imgCode:             (array) mapping btw "e","h","s","b" and actual images. 
## bgI:                 (environment,optional) If given, number of annoations in the Background are given in the table.
## fgI:                 (environment,optional) If given, number of annoations in the foregroundare given in the table.

makeGO.expandtable<- function(significant.terms,ON,groups,gInfo = FALSE,upimg = NULL,dwimg = NULL,co, imgCode = array(NA,0),bgI = bgI,fgI = fgI)
{
  ## scripts,
  getScripts <- function(nr.dataCol)
   {
     res <- "<style>\n.hidden  {display:none;}\n.shown   {dispaly:block;}\n.dataCol {list-style-type:none;\n margin-left:0px;\n margin:0px;\n padding:0px;\n text-align:left;}\n li { \n white-space:nowrap;\n }\n ul  { \n list-style-type:none;\n margin:0px;\n margin-left:0px;\n padding:0px;\n }\n </style>\n <script type=\"text/javaScript\">\n function hideChildren(node)\n { \n var nodes = node.childNodes;\n for(var i=0;i<nodes.length;i++)\n {\nif (typeof(nodes[i].tagName) != \"undefined\" && (nodes[i].tagName.toLowerCase() == 'li' || nodes[i].tagName.toLowerCase() == 'ul'))\n {\n if(nodes[i].className != 'hidden')\n{\n hideChildren(nodes[i]);\n}\n } \nif (typeof(nodes[i].tagName) != \"undefined\" && nodes[i].tagName.toLowerCase() == 'li')\n { \n nodes[i].className = 'hidden';\n (document.getElementById('info' + nodes[i].id)).className = 'hidden';\n(document.getElementById('col' + nodes[i].id)).className = 'hidden';\n"
     for (i in 1:nr.dataCol)
       {
         res <- c(res,paste("(document.getElementById('",letters[i],"' + nodes[i].id)).className = 'hidden';\n",sep =""))
       }
     res <- c(res," }\n }\n }\n \n function toggleRow(node)\n {\n var tmp = 'shown';\n if(node.className == 'shown')\n {\n hideChildren(node)\n tmp = 'hidden';\n }\n node.className = tmp;\n (document.getElementById('info' + node.id)).className = tmp;\n(document.getElementById('col' + node.id)).className = tmp;\n")
     for (i in 1:nr.dataCol)
       {
         res <- c(res,paste("(document.getElementById('",letters[i],"' + node.id)).className = tmp;\n",sep =""))
       }
     res <- c(res,"}\n function toggleChildren(nodeID)\n {\n var tmp = document.getElementById(nodeID);\n var nodes = tmp.childNodes;\n var licands;\n for (var i = 0;i<nodes.length;i++)\n { \n // only <ul>'s\n if(typeof(nodes[i].tagName) != \"undefined\" && nodes[i].tagName.toLowerCase() == 'ul')\n {\n licands = nodes[i].childNodes;\n for (var j = 0;j<licands.length;j++)\n { \n // only <li>'s\n if(typeof(licands[j].tagName) != \"undefined\" && licands[j].tagName.toLowerCase() == 'li')\n {\n toggleRow(licands[j]); \n }\n }\n }\n }\n }\n </script>\n")
     return(res)
   }

  toImg <- function(st)
    {
      tmp <- strsplit(st,"")[[1]]
      tmp[which(tmp == "s")] <- imgCode["s"]
      tmp[which(tmp == "h")] <- imgCode["h"]
      tmp[which(tmp == "e")] <- imgCode["e"]
      tmp[which(tmp == "b")] <- imgCode["b"]
      return(paste(tmp,collapse = ""))
    }

  chImgFix <- function(st)
    {
      tmp <- strsplit(st,"")[[1]]
      tmp[which(tmp == "s")] <- "h"
      tmp[which(tmp == "e")] <- "b"
      return(paste(tmp,collapse = ""))
    }

  
  ## helper, terms recursively down the GO-structure.
  ## node is the child we're about to expand. gidls are all the gids top-down in the <ul>-list.
  ## allterms are the ones in our set. 
   makeTermCol <- function(node,allterms,gidls,currid,imgcode)
    {
      ##print(imgcode)
      ulres      <- NULL
      level.kids <- NULL 
      try(level.kids <- get(node,SEGOCHILDREN),silent = TRUE)
      ## kids within the family
      internal.level.kids <- intersect(level.kids,allterms)
      if (length(internal.level.kids)>0)
        {
          gidls <- c(gidls,node)
          n.tmp <- ""
          n.tmp <- try(n.tmp <- get(node,SEGOTERM),silent = TRUE)
          substr(n.tmp,1,1) <- toupper(substr(n.tmp,1,1))
          ulres <- paste("<div onClick=\"toggleChildren('",currid,"')\">",toImg(imgcode[node]),"<b>",n.tmp,"</b></div><ul>\n",sep = "")
          ##ulres <- "<ul>\n" ##' node's <ul> not the kids. 
          for(i in 1:length(internal.level.kids))
            {
              imgAdd <- "s"
              if(i == length(internal.level.kids))
                {imgAdd <- "e"}
              imgcode[internal.level.kids[i]] <- paste(chImgFix(imgcode[node]),imgAdd,sep ="")
              ## this will give all nodes with children a 'onClick'. 
              currid  <- currid + 1
              ulres   <- c(ulres,paste("<li id = \"",currid,"\" class = 'hidden'>",sep =""))
              tmp     <- makeTermCol(internal.level.kids[i],allterms,NULL,currid,imgcode)
              currid  <- currid + length(tmp[[2]]) -1 ## compensate for the +1 above.
              ulres   <- c(ulres,tmp[[1]],"</li>\n")
              gidls   <- c(gidls,tmp[[2]])
              imgcode <- tmp[[3]]
            }
          ulres <- c(ulres,"</ul>\n") ##' node's <ul> not the kids.
        }else{
          n.tmp <- ""
          n.tmp <- try(n.tmp <- get(node,SEGOTERM),silent = TRUE)
          substr(n.tmp,1,1) <- toupper(substr(n.tmp,1,1))
          ulres <- paste(toImg(imgcode[node])," <span style=\cursor:crosshair;\">",n.tmp,"</span>\n",sep = "")
          gidls <- c(gidls,node)
        }
      return(list(ulres,gidls,imgcode))
    }
            
  ##
  ## MAIN:
  ##
  
  res <- "<table border>"
  nr.groups <- length(groups)
  res <- c(res,getScripts(nr.groups))
  ## headers & construnction of leaves.
  res <- c(res,paste("<tr><th colspan = \"",nr.groups+3,"\" align = center>Two sided pvalues (Click to expand)</th></tr>",sep = ""))
  tmpr <- "<tr><td>GO Term</td><td>Summary</td><td>GO ID</td>"
  all.sig <- NULL ## will contain all 'leaves'
  for (i in 1:nr.groups)
    {
      all.sig <- union(all.sig,names(significant.terms[[i]]$up))
      all.sig <- union(all.sig,names(significant.terms[[i]]$down))
      tmpr <- c(tmpr,"<td>",paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),"</td>")
    }
  tmpr <- c(tmpr,"</tr>\n")
  res <- c(res,paste(tmpr,collapse = ""))

  ## get all induced nodes and start the recursion.
  if (length(all.sig)>0)
    {
      ##print("continuing")
      topn <- topnode[[ON]]
      sparseTree <- getSparseTree(all.sig)
      ## the navigate column
      res <- c(res,"<tr>\n<td valign = \"top\">")
      imgcode <- array(NA,0)
      imgcode[topn] <- ""
      tmp <- makeTermCol(topn,sparseTree,NULL,0,imgcode)
      res <- c(res,paste("<ul style=\"cursor:pointer;\">\n<li id = \"0\">",sep =""),tmp[[1]],"</li></ul>\n") 
      res <- c(res,"</td>")
      ## the color column.
      res <- c(res,"<td valign =\"top\">\n")
      res <- c(res,"<ul class = 'dataCol' style = \"text-align:center;\">\n")
      c.tmp <- ""
      lbl <- "#63B8FF"
      bl  <- "#007FFF"
      rd  <- "#FF3300"
      lrd <- "#FF7F50"
      wh  <- "#FFFFFF"
      SKIP <- FALSE
      for(i in 1:length(groups))
        {
          pval <- significant.terms[[i]][["all"]][topn]
          ## if the background is empty at any time point, the results are NA
          if(is.na(pval))
            {
              pval <- "*"
              SKIP <- TRUE
              col  <- wh
            }else{
              if(abs(pval)<=co)
                {
                  col <- lrd
                  if(pval <0)
                    col <- lbl
                }else{
                  col <- lrd
                  if(pval <0)
                    col <- lbl
                }
            }
          c.tmp <- c(c.tmp,paste("<span style=\"width:9px;height:9px;background-color:",col,";\">",i,"</span>",sep = ""))
        }
      res <- c(res,paste("<li id = \"",paste("col0",sep = ""),"\">",paste(c.tmp,collapse = ""),"</li>\n",sep = ""))
      for (j in 2:length(tmp[[2]]))
        {
          c.tmp <- ""
          for(i in 1:length(groups))
            {
              pval <- significant.terms[[i]][["all"]][tmp[[2]][j]]
              if(is.na(pval))
                {
                  pval <- "*"
                  SKIP <- TRUE
                  col  <- wh
                }else{
                  if(abs(pval)<=co)
                    {
                      col <- lrd
                      if(pval <0)
                        col <- lbl
                    }else{
                      col <- lrd
                      if(pval <0)
                    col <- lbl
                    }
                }
              c.tmp <- c(c.tmp,paste("<span style=\"width:9px;height:9px;background-color:",col,";\">",i,"</span>",sep = ""))
            }
          res <- c(res,paste("<li class = 'hidden' id = \"",paste("col",j-1,sep = ""),"\">",paste(c.tmp,collapse = ""),"</li>\n",sep = ""))
        }
      res <- c(res,"</ul>\n")
      res <- c(res,"</td>\n")
      ## the info column.
      res <- c(res,"<td valign =\"top\">\n")
      res <- c(res,"<ul class = 'dataCol' style = \"text-align:leftt;\">\n")
      gTe <- ""
      ##gTe <- try(gTe <- get(tmp[[2]][1],SEGOTERM),silent = TRUE)
      gTe <- paste("<a target = \"_blank\" href = http://www.ebi.ac.uk/ego/DisplayGoTerm?id=",tmp[[2]][1],">",tmp[[2]][1],"</a>",sep = "")
      if(!is.null(fgI))
        {
          nr.a <- sum(mget(tmp[[2]][1],fgI,ifnotfound = 0)[[1]],na.rm = TRUE)
          gTe <- paste(gTe," (",nr.a,")",sep = "")
        }
      res <- c(res,"<li id = \"",paste("info0",sep = ""),"\">",gTe,"</li>\n")
      for (i in 2:length(tmp[[2]]))
        {
          gTe <- ""
          ##gTe <- try(gTe <- get(tmp[[2]][i],SEGOTERM),silent = TRUE)
          gTe <- paste("<a target = \"_blank\" href = http://www.ebi.ac.uk/ego/DisplayGoTerm?id=",tmp[[2]][i],">",tmp[[2]][i],"</a>",sep = "")
          if(!is.null(fgI))
            {
              nr.a <- sum(mget(tmp[[2]][i],fgI,ifnotfound = 0)[[1]],na.rm = TRUE)
              gTe <- paste(gTe," (",nr.a,")",sep = "")
            }
          res <- c(res,paste("<li class = 'hidden' id = \"",paste("info",i-1,sep = ""),"\">",gTe,"</li>\n",sep = ""))

        }
      res <- c(res,"</ul>\n")
      res <- c(res,"</td>\n")
      
      ## the p-value columns
      for(i in 1:length(groups))
        {
          pic.tmp <- "" ## up/dwn
          res <- c(res,"<td valign =\"top\">\n")
          res <- c(res,"<ul class = 'dataCol'>\n")
          pval <- significant.terms[[i]][["all"]][topn]
          if(is.na(pval))
            {
              pval  <- "*"
              tmp.p <- pval
              SKIP  <- TRUE
            }else{
              if (abs(pval) <= co)
                {
                  pic.tmp <- ""
                  col <- "red"
                  if(pval <= 0)
                    {
                      col <- "blue"
                    }
                  tmp.p <- paste("<font color =\"",col,"\">",abs(pval),"</font>",sep = "")
                  if(gInfo)
                    {
                      tmp.p <- paste(pic.tmp,"<a href = \"geneInfo",ON,paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),
                                     ".html#",topn,"\" target = \"sub_browser\">",tmp.p,"</a>",sep = "")
                    }
                }else{
                  if (!is.null(dwimg) && !is.null(upimg))
                    {
                      if(pval < 0)
                        pic.tmp <- paste("<img src =\"",dwimg,"\">",sep  = "")
                      else
                        pic.tmp <- paste("<img src =\"",upimg,"\">",sep  = "")
                    }
                  tmp.p <- paste(pic.tmp," ",abs(pval),sep = "")
                }
            }
          if(!is.null(fgI))
            {
              nr.a <- sum(mget(topn,fgI,ifnotfound = 0)[[1]][groups[[i]]])
              tmp.p <- paste(tmp.p," (",nr.a,")",sep = "")
            }
          res <- c(res,"<li id = \"",paste(letters[i],"0",sep = ""),"\">",tmp.p,"</li>\n")
          for (j in 2:length(tmp[[2]]))
            {
              pval <- significant.terms[[i]][["all"]][tmp[[2]][j]]
              if(is.na(pval))
                {
                  pval  <- "*"
                  tmp.p <- pval
                  SKIP  <- TRUE
                }else{
                  if (abs(pval) <= co)
                    {
                      pic.tmp <- ""
                      col <- "red"
                      if(pval <= 0)
                        {
                          col <- "blue"
                        }
                      tmp.p <- paste("<font color =\"",col,"\">",abs(pval),"</font>",sep = "")
                      if(gInfo)
                        {
                          tmp.p <- paste(pic.tmp,"<a href = \"geneInfo",ON,paste("[",paste(groups[[i]],collapse = ","),"]",sep = ""),
                                         ".html#",tmp[[2]][j],"\" target = \"sub_browser\">",tmp.p,"</a>",sep = "")
                        }
                    }else{
                      if (!is.null(dwimg) && !is.null(upimg))
                        {
                          if(pval < 0)
                            pic.tmp <- paste("<img src =\"",dwimg,"\">",sep  = "")
                          else
                            pic.tmp <- paste("<img src =\"",upimg,"\">",sep  = "")
                        }
                      tmp.p <- paste(pic.tmp," ",abs(pval),sep = "")
                    }
                }
              if(!is.null(fgI))
                {
                  nr.a <- sum(mget(tmp[[2]][j],fgI,ifnotfound = 0)[[1]][groups[[i]]],na.rm = TRUE)
                  tmp.p <- paste(tmp.p," (",nr.a,")",sep = "")
                }
              res <- c(res,paste("<li class = 'hidden' id = \"",paste(letters[i],j-1,sep = ""),"\">",tmp.p,"</li>\n",sep = ""))
            }
          res <- c(res,"</ul>\n")
          res <- c(res,"</td>\n")
        }
      res <- c(res,"</table>")
      if(SKIP)
        res <- c(res,"<br><b>* These terms were not used in the background for this timepoint. The p-values cannot be calculated.</b><br>")
    }else{
      res <- c(res,paste("<tr><td colspan =\"",length(groups) +3,
                         "\" align = \"center\">No significant results.</td></tr></table>",sep = ""))
    }
  return(res)
}





## fnc, description
##
## Notes: possible side effects & such
##
## Todo: what could be improved.
##
## Parameters:
## name:     (type,constraints) Description.
