###########################################################################
##
## %File: stats.R
## %Plug-in: Miscellaneous: log2-ratio statistics
## %Version: R2:1.0
## 
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 28/11 2005
## %Last modified: 26/7 2006
##
## %Description: his is a simple plug-in for obtaining some statistics over
##  log2-ratios. It computes mean+n*stdev, for the log2-ratio on each array.
##  The main use of this plug-in is to obtain information about where to place
##  a log2-ratio cut-off in order to get signals that are more/less than n
##  standard deviations above/below the mean.
##
## %Output: The plug-in creates a table containing the resulting statistics.
##
## %Parameters:
##  - n: A comma separated string with (positive or negative) numbers
##
## %Screen shots: log2.png (400)
##
#############################################################################

## Ignore warnings
options(warn=-1)

source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))

## Load data and parameters
pars<-read.parameters()
expr<-read.experiment()
data<-read.data()

stdevs <- as.numeric(unlist(strsplit(pars$stdevs,",")))

nrAssays <- length(data)

stdevTexts<-as.character(stdevs)

stdevTexts[which(stdevs >= 0)] <- paste("+",stdevTexts[which(stdevs >= 0)], sep="")

res <- c(c("Array", "Mean", "sd"),paste("Mean",stdevTexts,"*sd",sep=""))

debug.output("Data and parameters loaded")

for (i in 1:nrAssays){

  ## Get the reporter Ids
  repIds<-data[[i]]$reporterId
  Mvalues <- log2(as.numeric(data[[i]]$intensity1)) - log2(as.numeric(data[[i]]$intensity2))
  res <- rbind(res,c(data[[i]]$assayName,
                     signif(mean(Mvalues),digits = 4),
                     signif(sd(Mvalues),digits = 4),
                     c(signif(mean(Mvalues) + stdevs*sd(Mvalues),digits = 4))))
}

debug.output("All stats computed")

## ###########################
##
## Create HTML result page 
##
## ###########################

outfile <- paste("Results.html", sep="")

cat("<html>\n", file=outfile)
cat(styleStr, file=outfile, append=TRUE)

cat(paste("<h1 align=center>log2-ratio statistics</h1>",sep=""), file=outfile, append=TRUE)

cat("<table width=600 align=\"center\" border=\"1\" frame=\"border\" rules=\"none\"><tr bgcolor=\"ddddff\"><td>", file=outfile, append=TRUE)

write.table(res,file=outfile, eol="</td><tr bgcolor=\"f0f0ff\">\n<td>",sep="</td><td>",append=TRUE, quote = FALSE, row.names = FALSE, col.names = FALSE)

cat("</table></body></html>", file=outfile, append=TRUE)

debug.output("Finished")

clean.up()

